require "tq_curency_manager/version"

module TqCurencyManager
  class Money

  	# Init & getters

  	USD_FOR_EUR = 1.11
  	BITCOIN_FOR_EUR = 0.0047
  	@@rates = {
  		'EUR' => 1,
  		'USD' => USD_FOR_EUR,
  		'Bitcoin' => BITCOIN_FOR_EUR
  	}

  	def initialize amount, curency
  		@amount = amount.to_f
  		@curency = curency
  		if !@@rates.key? curency
  			raise ArgumentError.new("Not valid curency")
  		end
  	end

  	def self.rates
  		@@rates
  	end

  	def amount
  		# we just give different rounding with bitcoin
  		if curency == 'Bitcoin'
  			@amount.round(8)
  		else
  			@amount.round(2)
  		end
  	end

  	def curency
  		@curency
  	end

  	# '%.Xf' % allows to have X digits after the point  
  	def inspect
  		if curency == 'Bitcoin'
  			'%.8f' % amount + " " + curency
  		else
  			'%.2f' % amount + " " + curency
  		end
  	end

  	# Setters

  	def self.conversion_rates base_curency, rates
  		@@rates[base_curency] = 1
  		rates.each do |k,v|
  			@@rates[k] = v.to_f
  		end
  		@@rates
  	end

  	# Curency conversion

  	def convert_to(curency)
  		if @@rates[curency]
  			self.class.new(amount_convert(curency), curency)
  		else
  			puts 'error : you have to initialize the curency convertion for ' + curency
  			nil
  		end
  	end

  	# Comparaison methods

  	 	def ==(instance)
  	 		self.convert_to('EUR').inspect == instance.convert_to('EUR').inspect
  	 	end

  	 	def +(instance)
  	 		amount = self.amount + instance.convert_to(self.curency).amount
  	 		self.class.new(amount, self.curency)
  	 	end

  	 	def -(instance)
  	 		amount = self.amount - instance.convert_to(self.curency).amount
  	 		self.class.new(amount, self.curency)
  	 	end

  	 	def *(float)
  	 		amount = self.amount * float
  	 		self.class.new(amount, self.curency)
  	 	end

  	 	def /(float)
  	 		amount = self.amount / float
  	 		self.class.new(amount, self.curency)
  	 	end

  	 	def >(instance)
  	 		amount = instance.convert_to(self.curency).amount
  	 		self.amount > amount
  	  end

			def <(instance)
				amount = instance.convert_to(self.curency).amount
				self.amount < amount
		 	end

  	private

  		def amount_convert(new_curency)
  			curency_rate = @@rates[curency]
  			new_curency_rate = @@rates[new_curency]
  			amount = @amount.to_f

  			((amount * ( new_curency_rate / curency_rate )))
  		end

  end
end
